/***********************************************************************************
 *
 * Copyright (c) 2013 Louka Desroziers
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#import "UIImage+AutomaticCapInsets.h"

#if !__has_feature(objc_arc)
#error This category requires automatic reference counting
#endif

@interface NSString (PXPAutomaticCapInsets)
+ (NSRange)__rangeOfEdgeInsetsFromString:(NSString *)string;
+ (UIEdgeInsets)__edgeInsetsFromString:(NSString *)string;
+ (NSString *)__stringByRemovingEdgeInsetsFromString:(NSString *)string;

- (NSRange)__rangeOfEdgeInsets;
- (UIEdgeInsets)__edgeInsets;
- (NSString *)__stringByRemovingEdgeInsets;
@end
@implementation NSString (PXPAutomaticCapInsets)

+ (NSRange)__rangeOfEdgeInsetsFromString:(NSString *)string
{
    NSRange rangeForOpenBrace = [string rangeOfString:@"{"];
    NSRange rangeForCloseBrace = [string rangeOfString:@"}"];
    
    if(rangeForOpenBrace.location != NSNotFound
       && rangeForCloseBrace.location != NSNotFound)
        return NSMakeRange(rangeForOpenBrace.location, rangeForCloseBrace.location - rangeForOpenBrace.location +1);
    
    return NSMakeRange(NSNotFound, 0);
}

+ (UIEdgeInsets)__edgeInsetsFromString:(NSString *)string
{
    NSRange rangeOfEdgeInsets = [[self class] __rangeOfEdgeInsetsFromString:string];
    if(rangeOfEdgeInsets.location != NSNotFound)
        return UIEdgeInsetsFromString([string substringWithRange:rangeOfEdgeInsets]);

    return UIEdgeInsetsZero;
}

+ (NSString *)__stringByRemovingEdgeInsetsFromString:(NSString *)string
{
    NSRange rangeOfEdgeInsets = [[self class] __rangeOfEdgeInsetsFromString:string];
    
    if(rangeOfEdgeInsets.location != NSNotFound)
        return [string stringByReplacingCharactersInRange:rangeOfEdgeInsets withString:@""];
    
    return string;
}

- (NSRange)__rangeOfEdgeInsets {
    return [[self class] __rangeOfEdgeInsetsFromString:self];
}

- (UIEdgeInsets)__edgeInsets {
    return [[self class] __edgeInsetsFromString:self];
}

- (NSString *)__stringByRemovingEdgeInsets {
    return [[self class] __stringByRemovingEdgeInsetsFromString:self];
}


@end

@interface NSFileManager (PXPAutomaticCapInsets)
- (NSString *)__capInsetsPathForFilename:(NSString *)filename inDirectory:(NSString *)directoryPath;
@end
@implementation NSFileManager (PXPAutomaticCapInsets)

- (NSString *)__capInsetsPathForFilename:(NSString *)filename inDirectory:(NSString *)directoryPath
{
    // Loop on each resources in the given directoryPath until it found the capInsets corresponding file.
    NSArray                 *allowedExtensions = @[@"png", @"jpg", @"jpeg", @"tif", @"tiff", @"gif"];
    NSString                *capInsetsPath = nil;
    
    for(NSString *resourceName in [self contentsOfDirectoryAtPath:directoryPath error:nil])
    {
        if([allowedExtensions containsObject:[[resourceName pathExtension] lowercaseString]]
           && UIEdgeInsetsEqualToEdgeInsets([resourceName __edgeInsets], UIEdgeInsetsZero) == NO
           && [[resourceName __stringByRemovingEdgeInsets] isEqualToString:filename])
        {
            capInsetsPath = [directoryPath stringByAppendingPathComponent:resourceName];
            break;
        }
    }
    
    return capInsetsPath;
}

@end

@implementation NSBundle (PXPAutomaticCapInsets)

- (NSString *)pathForResizableResource:(NSString *)name ofType:(NSString *)ext {
    return [self pathForResizableResource:name ofType:ext inDirectory:nil];
}

- (NSString *)pathForResizableResource:(NSString *)name ofType:(NSString *)ext inDirectory:(NSString *)subpath;
{
    NSString *directoryPath = [self resourcePath];
    if(subpath != nil)
        directoryPath = [directoryPath stringByAppendingPathComponent:subpath];
    
    NSString *filePath = directoryPath;
    NSString *extension = (ext == nil ? [name pathExtension] : ext);
    if(name != nil)
        filePath = [filePath stringByAppendingPathComponent:[name stringByDeletingPathExtension]];
    if(extension != nil)
        filePath = [filePath stringByAppendingPathExtension:extension];
    
    if([filePath __rangeOfEdgeInsets].location != NSNotFound) {
        NSLog(@"warning: You provided the capInsets in your resource name/path. This is quite stupid.");
        return filePath;
    }
    
    return [[NSFileManager defaultManager] __capInsetsPathForFilename:[filePath lastPathComponent]
                                                          inDirectory:directoryPath];
}

@end

@implementation UIImage (PXPAutomaticCapInsets)

+ (UIImage *)resizableImageNamed:(NSString *)name {
    return [[self class] resizableImageNamed:name forMode:UIImageResizingModeTile];
}

+ (UIImage *)resizableImageNamed:(NSString *)name forMode:(UIImageResizingMode)resizingMode
{
    NSString        *directoryPath = [[NSBundle mainBundle] resourcePath];
    NSString        *resizableImagePath = [[NSFileManager defaultManager] __capInsetsPathForFilename:name
                                                                                         inDirectory:directoryPath];
    NSString        *resizableImageName = [resizableImagePath lastPathComponent];
    UIEdgeInsets    edgeInsets = [resizableImageName __edgeInsets];
    
    return [[[self class] imageNamed:resizableImageName] resizableImageWithCapInsets:edgeInsets
                                                                        resizingMode:resizingMode];
}


+ (UIImage *)resizableImageWithContentsOfFile:(NSString *)path {
    return [[self class] resizableImageWithContentsOfFile:path forMode:UIImageResizingModeTile];
}

+ (UIImage *)resizableImageWithContentsOfFile:(NSString *)path forMode:(UIImageResizingMode)resizingMode {
    return [[[self class] alloc] initWithResizableContentsOfFile:path forMode:resizingMode];
}


- (id)initWithResizableContentsOfFile:(NSString *)path{
    return [self initWithResizableContentsOfFile:path forMode:UIImageResizingModeTile];
}

- (id)initWithResizableContentsOfFile:(NSString *)path forMode:(UIImageResizingMode)resizingMode
{
    // The developer must call this method using -pathForResizableResource:ofType: from NSBundle.
    NSString        *resizableImageName = [path lastPathComponent];
    UIEdgeInsets    edgeInsets = [resizableImageName __edgeInsets];
    
    return [[self initWithContentsOfFile:path] resizableImageWithCapInsets:edgeInsets
                                                              resizingMode:resizingMode];
}


@end
